@@ -13,6 +13,7 @@ import (
 	"os"
 	"path/filepath"
 	"strconv"
+	"syscall"
 	"testing"
 	"time"
 
@@ -3140,3 +3141,130 @@ func TestBuildStageMetricsFailBuild(t *testing.T) {
 
 	assert.Len(t, stagesMap, 0)
 }
+
+func TestBuildDurationsAndBoundaryTimes(t *testing.T) {
+	rc := new(RunnerConfig)
+	rc.RunnerSettings.Executor = t.Name()
+
+	build, err := NewBuild(JobResponse{}, rc, nil, nil)
+	require.NoError(t, err)
+
+	startedAt1 := build.StartedAt()
+
+	assert.False(t, startedAt1.IsZero(), "StartedAt should not be a zero-time")
+	assert.True(t, build.FinishedAt().IsZero(), "FinishedAt should be a zero-time")
+
+	time.Sleep(10 * time.Millisecond)
+	currentDuration1 := build.CurrentDuration()
+	assert.True(t, currentDuration1 >= 10*time.Millisecond, "Current job duration should be greater tha 10ms")
+
+	time.Sleep(10 * time.Millisecond)
+	currentDuration2 := build.CurrentDuration()
+	assert.True(t, currentDuration2 >= 20*time.Millisecond, "Current job duration should be greater tha 20ms")
+	assert.NotEqual(t, currentDuration1, currentDuration2, "Subsequent CurrentDuration() values shouldn't be equal")
+
+	time.Sleep(10 * time.Millisecond)
+	assert.Equal(t, time.Duration(-1), build.FinalDuration(), "If ensureFinishedAt() wasn't called, final duration should be equal to -1")
+
+	// Mark the job as finished!
+	build.ensureFinishedAt()
+
+	finalDuration1 := build.FinalDuration()
+	finishedAt1 := build.FinishedAt()
+	assert.True(t, finalDuration1 >= 30*time.Millisecond, "Final duration should be greater than 30ms")
+	assert.False(t, finishedAt1.IsZero(), "FinishedAt should not be a zero-time")
+
+	time.Sleep(10 * time.Millisecond)
+	startedAt2 := build.StartedAt()
+	finishedAt2 := build.FinishedAt()
+	finalDuration2 := build.FinalDuration()
+
+	assert.Equal(t, finalDuration1, finalDuration2, "Subsequent FinalDuration() values should be equal")
+	assert.Equal(t, finishedAt1, finishedAt2, "FinishedAt() should not change")
+	assert.Equal(t, startedAt1, startedAt2, "StartedAt() should not change")
+}
+
+func TestBuild_runCallsEnsureFinishedAt(t *testing.T) {
+	tests := map[string]struct {
+		mockRun     func(t *testing.T, cancelFn func(error), interrupt chan os.Signal, executor *MockExecutor)
+		assertError func(t *testing.T, err error)
+	}{
+		"succeeded job": {
+			mockRun: func(t *testing.T, _ func(error), _ chan os.Signal, executor *MockExecutor) {
+				executor.EXPECT().Run(mock.Anything).Return(nil)
+			},
+		},
+		"failed job": {
+			mockRun: func(t *testing.T, _ func(error), _ chan os.Signal, executor *MockExecutor) {
+				executor.EXPECT().Run(mock.Anything).Return(assert.AnError)
+			},
+			assertError: func(t *testing.T, err error) {
+				assert.ErrorIs(t, err, assert.AnError)
+			},
+		},
+		"canceled job": {
+			mockRun: func(t *testing.T, cancelFn func(error), _ chan os.Signal, executor *MockExecutor) {
+				executor.EXPECT().Run(mock.Anything).RunAndReturn(func(command ExecutorCommand) error {
+					cancelFn(assert.AnError)
+					time.Sleep(10 * time.Millisecond)
+					return nil
+				}).Once()
+				executor.EXPECT().Run(mock.Anything).Return(nil)
+			},
+			assertError: func(t *testing.T, err error) {
+				assert.ErrorIs(t, err, assert.AnError)
+			},
+		},
+		"interrupted runner process": {
+			mockRun: func(t *testing.T, _ func(error), interrupt chan os.Signal, executor *MockExecutor) {
+				executor.EXPECT().Run(matchBuildStage(BuildStagePrepare)).RunAndReturn(func(command ExecutorCommand) error {
+					interrupt <- syscall.SIGHUP
+					time.Sleep(10 * time.Millisecond)
+					return nil
+				}).Once()
+				executor.EXPECT().Run(mock.Anything).Return(nil)
+			},
+			assertError: func(t *testing.T, err error) {
+				var eerr *BuildError
+				if assert.ErrorAs(t, err, &eerr) {
+					assert.Equal(t, RunnerSystemFailure, eerr.FailureReason)
+					assert.Contains(t, eerr.Inner.Error(), "aborted: hangup")
+				}
+			},
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			rc := new(RunnerConfig)
+			rc.RunnerSettings.Executor = t.Name()
+
+			interrupt := make(chan os.Signal, 1)
+
+			build, err := NewBuild(JobResponse{}, rc, interrupt, nil)
+			require.NoError(t, err)
+
+			ctx, cancelFn := context.WithCancelCause(context.Background())
+			defer cancelFn(nil)
+
+			require.Zero(t, build.finishedAt)
+
+			executor := NewMockExecutor(t)
+			executor.EXPECT().Shell().Return(&ShellScriptInfo{Shell: "script-shell"})
+			require.NotNil(t, tt.mockRun)
+			tt.mockRun(t, cancelFn, interrupt, executor)
+
+			trace := NewMockJobTrace(t)
+			trace.EXPECT().SetCancelFunc(mock.AnythingOfType("context.CancelFunc")).Maybe()
+
+			err = build.run(ctx, trace, executor)
+			if tt.assertError != nil {
+				tt.assertError(t, err)
+			} else {
+				assert.NoError(t, err)
+			}
+
+			assert.NotZero(t, build.finishedAt)
+		})
+	}
+}
